/***********************************************************************************************************************
 *
 * BlueBook - use live code examples in your book
 * Copyright (C) 2011-2011 by Tidalwave s.a.s.
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebook.java.net
 * SCM: https://hg.java.net/hg/bluebook~src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebook.plugin.maven;

import lombok.ToString;
import java.util.List;
import org.apache.maven.artifact.factory.ArtifactFactory;
import org.apache.maven.artifact.metadata.ArtifactMetadataSource;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.resolver.ArtifactResolver;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import it.tidalwave.bluebook.configuration.Configuration;
import it.tidalwave.bluebook.configuration.ConfigurationProvider;
import java.io.File;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 * @goal generate-filtered-source
 * @threadSafe
 *
 **********************************************************************************************************************/
@ToString
public class BlueBookMojo extends AbstractMojo
  {
    /*******************************************************************************************************************
     *
     * @parameter 
     * @readonly
     * 
     ******************************************************************************************************************/
    private String groupId;
    
    /*******************************************************************************************************************
     *
     * @parameter
     * @readonly
     * 
     ******************************************************************************************************************/
    private String artifactId;
    
    /*******************************************************************************************************************
     *
     * @parameter
     * @readonly
     * 
     ******************************************************************************************************************/
    private String version;
    
    /*******************************************************************************************************************
     *
     * @parameter default-value="sources"
     * @readonly
     * 
     ******************************************************************************************************************/
    private String classifier;
    
    /*******************************************************************************************************************
     *
     * @parameter default-value="jar"
     * @readonly
     * 
     ******************************************************************************************************************/
    private String type;
    
    /*******************************************************************************************************************
     *
     * @parameter
     * @readonly
     * 
     ******************************************************************************************************************/
    private String fileName;
    
    /*******************************************************************************************************************
     *
     * @parameter default-value="true"
     * @readonly
     * 
     ******************************************************************************************************************/
    private boolean skipHeader;
    
    /*******************************************************************************************************************
     *
     * @parameter default-value="true"
     * @readonly
     * 
     ******************************************************************************************************************/
    private boolean skipBlockComments;
    
    /*******************************************************************************************************************
     *
     * @parameter default-value="false"
     * @readonly
     * 
     ******************************************************************************************************************/
    private boolean skipImports;
    
    /*******************************************************************************************************************
     *
     * @parameter default-value="false"
     * @readonly
     * 
     ******************************************************************************************************************/
    private boolean skipPackage;
    
    /*******************************************************************************************************************
     *
     * @parameter 
     * @readonly
     * 
     ******************************************************************************************************************/
    private String includedFragment;

    /*******************************************************************************************************************
     *
     * @parameter 
     * @readonly
     * 
     ******************************************************************************************************************/
    private String excludedFragment;

    /*******************************************************************************************************************
     *
     * @parameter
     * @readonly
     * 
     ******************************************************************************************************************/
    private String outputFile;
    
    /*******************************************************************************************************************
     *
     * @parameter expression=
     *  "${component.org.apache.maven.artifact.factory.ArtifactFactory}"
     * @required
     * @readonly
     * 
     ******************************************************************************************************************/
    protected ArtifactFactory factory;

    /*******************************************************************************************************************
     *
     * @parameter expression="${component.org.apache.maven.artifact.resolver.ArtifactResolver}"
     * @required
     * @readonly
     * 
     ******************************************************************************************************************/
    protected ArtifactResolver artifactResolver;

    /*******************************************************************************************************************
     *
     * List of Remote Repositories used by the resolver
     * 
     * @parameter expression="${project.remoteArtifactRepositories}"
     * @readonly
     * @required
     * 
     ******************************************************************************************************************/
    protected List remoteRepositories;

    /*******************************************************************************************************************
     *
     * Location of the local repository.
     * 
     * @parameter expression="${localRepository}"
     * @readonly
     * @required
     * 
     ******************************************************************************************************************/
    protected ArtifactRepository localRepository;
    
    /*******************************************************************************************************************
     *
     * @parameter default-value="${project}"
     * 
     ******************************************************************************************************************/
    protected MavenProject mavenProject;
    
    /*******************************************************************************************************************
     *
     * @parameter default-value="${project.build.directory}"
     * 
     ******************************************************************************************************************/
    protected File buildDirectory;
    
    /*******************************************************************************************************************
     *
     * @parameter default-value="${project.artifactMetadataSource}"
     * 
     ******************************************************************************************************************/
    protected ArtifactMetadataSource artifactMetadataSource;
    
    /*******************************************************************************************************************
     *
     * @parameter default-value="${basedir}/src/bluebook/bluebook.xml"
     * 
     ******************************************************************************************************************/
    protected String configurationFile;
    
    /*******************************************************************************************************************
     *
     * TODO: extract an interface, make it a mojo parameter
     * 
     ******************************************************************************************************************/
    protected ProgramListingTemplate template = new ProgramListingTemplate();
    
    // TODO: make it injectable and configurable
    private  ConfigurationProvider configurationProvider = new ConfigurationProvider();
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     * 
     ******************************************************************************************************************/
    @Override
    public void execute() 
      throws MojoExecutionException
      {
        final java.io.File configFile = new java.io.File(configurationFile);
        
        if (configFile.exists() && artifactId == null)
          {
            getLog().info("Configuration file " + configFile);
            final MavenConfigurationVisitor mcv = new MavenConfigurationVisitor(getLog(), 
                                                                                factory, 
                                                                                artifactResolver, 
                                                                                remoteRepositories, 
                                                                                localRepository, 
                                                                                mavenProject, 
                                                                                buildDirectory,
                                                                                artifactMetadataSource, 
                                                                                template);
            try  
              {
                final Configuration configuration = configurationProvider.loadConfiguration(configFile);
                configuration.accept(mcv);
              } 
            catch (Exception e)
              {
                throw new MojoExecutionException("", e);
              }
          }
        
        else
          {
            legacy_1_0(); // FIXME: temporarily support the old configuration    
          }
      }
    
    /*******************************************************************************************************************
     *
     * 
     ******************************************************************************************************************/
    private void legacy_1_0()
      throws MojoExecutionException
      {
        final MavenConfigurationVisitor mcv = new MavenConfigurationVisitor(getLog(), 
                                                                            factory, 
                                                                            artifactResolver, 
                                                                            remoteRepositories, 
                                                                            localRepository, 
                                                                            mavenProject, 
                                                                            buildDirectory,
                                                                            artifactMetadataSource, 
                                                                            template);
        mcv.setGroupId(groupId);
        mcv.setArtifactId(artifactId);
        mcv.setVersion(version);
        mcv.setClassifier(classifier);
        mcv.setType(type);
        mcv.setFileName(fileName);
        mcv.setSkipHeader(skipHeader);
        mcv.setSkipBlockComments(skipBlockComments);
        mcv.setSkipImports(skipImports);
        mcv.setSkipPackage(skipPackage);
        mcv.setIncludedFragments(includedFragment);
        mcv.setExcludedFragments(excludedFragment);
        mcv.setOutputDirectory("");
        mcv.setOutputFile(outputFile);
        
        mcv.extractFile();
      }
  }
